#------------------------------------------------------------------------------
#
#   Interesting Times - Actor
#
#------------------------------------------------------------------------------

from pygame import Rect
from albow.resource import get_image
from vectors import vector, as_vector
import constants as k

IDLE_MODE = 0
STUCK_MODE = -1

#------------------------------------------------------------------------------

class Actor(object):
	#  team         Team
	#  active       boolean
	#  position     vector
	#  cell         Vector    valid during think()

	is_reporter = False
	is_runner = False
	steps_per_minute = 8.0
	selection_image = get_image("actor_selection.png")
	selection_offset = (22, 35)
	hit_rect = Rect(-13, -30, 26, 31)
	
	team_no = property(lambda self: self.team.team_no)
	city = property(lambda self: self.team.city)
	level = property(lambda self: self.team.level)
	
	target_cell = None
	delay = 0
	
	on_duty = property(lambda self: self.active and self.paid)

	def __init__(self, team, cell):
		self.team = team
		self.position = as_vector(cell) * k.CELL_SIZE + k.CELL_SIZE // 2
		self.velocity = vector(0, 0)
		self.step_count = 0
		self.mode = IDLE_MODE
		self.active = False
		self.paid = True

	def draw(self, surf, selected):
		if self.on_duty:
			image = self.images[self.team_no - 1]
			surf.blit(image, self.position - self.foot_offset)
			if selected:
				surf.blit(self.selection_image, self.position - self.selection_offset)

	def begin_frame(self):
		if self.on_duty:
			cs = k.CELL_SIZE
			cs0 = cs[0]
			if self.step_count >= cs0:
				self.cell = self.position // cs
				self.think()
				self.step_count = 0 #self.steps_between_thinking
			steps = int(round(self.steps_per_minute * self.level.minutes_per_tick))
			assert cs0 % steps == 0, "Game speed does not result in valid actor step size"
			self.step_count += steps
			#if not self.delay:
			self.position = self.position + self.velocity * steps
	
	def get_hit_rect(self):
		r = Rect(self.hit_rect)
		return r.move(self.position)

	def set_goal(self, cell):
		pass

	def get_stuck(self):
		print self.__class__.__name__, "is stuck!" ###
		self.mode = STUCK_MODE
		self.velocity = k.zero

	def stop(self):
		self.mode = IDLE_MODE
		self.velocity = k.zero
