#------------------------------------------------------------------------------
#
#   Interesting Times - Runner
#
#------------------------------------------------------------------------------

import random
from random import choice
from albow.resource import get_image
from vectors import vector, as_vector, rotate_clockwise, rotate_anticlockwise
from actor import Actor, IDLE_MODE
from it_collections import CellSet, CellDict
import constants as k

vector_to_route_marker = CellDict()
vector_to_route_marker[0, 1] = 0
vector_to_route_marker[1, 0] = 1
vector_to_route_marker[0, -1] = 2
vector_to_route_marker[-1, 0] = 3

WALK_MODE = 1

#------------------------------------------------------------------------------

class Runner(Actor):
	#  city         City
	#  news_value   int
	#  paper_count  int
	#  initial_position   cell
	#  following_markers  boolean
	#  route_markers      CellDict --> int

	images = [
		get_image("runner-blue.png", border = 2),
		get_image("runner-red.png", border = 2),
	]
	
	is_runner = True
	foot_offset = vector(16, 30)
	
	visited_cells = None
	backtracked_cells = None

	def __init__(self, team, cell):
		Actor.__init__(self, team, cell)
		self.initial_position = self.position
		self.route_markers = CellDict()
		self.following_markers = False
	
	def get_route_marker(self, cell):
		return self.route_markers.get(cell)
	
	def set_route_marker(self, cell, marker):
		markers = self.route_markers
		if marker is not None:
			markers[cell] = marker
		else:
			if cell in markers:
				del markers[cell]
	
	def begin_day(self, news_value):
		print "Runner.begin_day" ###
		self.position = self.initial_position
		self.news_value = news_value
		#auto_play = self.team.auto_play
		auto_play = self.team.team_no == k.RED_TEAM
		if not auto_play or self.visited_cells is None:
			self.reset_memory()
		self.active = True
		self.mode = IDLE_MODE
	
	def reset_memory(self):
		self.visited_cells = CellSet()
		self.backtracked_cells = CellSet()
	
	def think(self):
		#print "Runner.think" ###
		self.attempt_delivery()
		if self.mode == IDLE_MODE:
			self.mode = WALK_MODE
			self.find_initial_walking_direction()
			print "Runner for team", self.team_no, "picked walking direction", self.velocity ###
		if self.mode == WALK_MODE:
			self.find_next_walking_direction()
	
	def find_initial_walking_direction(self):
		#  Try to find the edge of the road and start walking along it
		here = self.cell
		city = self.city
		for d in k.all_directions:
			#print "Runner thinks: Is there a road at", d, "from", here, "?" ###
			tile = city.tile_at(here + d)
			#print "Runner found", tile, "there" ###
			if not tile.is_road:
				#print "Runner thinks: No, there isn't." ###
				self.velocity = rotate_clockwise(d)
				return
		#  Road in all directions
		#self.get_stuck()
		self.velocity = choice(k.all_directions)

	def find_next_walking_direction(self):
		#  Try to stay on the road and follow the left edge
		here = self.cell 
		city = self.city
		marker = self.get_route_marker(here)
		if marker is not None:
			self.velocity = k.all_directions[marker]
			self.following_markers = True
		if self.following_markers:
			if city.tile_at(here + self.velocity).is_road:
				return
			else:
				self.following_markers = False
		ahead = self.velocity
		#print "Runner thinks: I'll try to follow the left edge from", here ###
		road_directions = []
		for d in direction_alternatives(ahead):
			#print "Runner thinks: Is there a road at %s?" % d
			if city.tile_at(here + d).is_road:
				#print "Runner thinks: Yes, there is" ###
				road_directions.append(d)
		for d in road_directions:
			cell = here + d
			if cell not in self.visited_cells:
				self.visited_cells.add(cell)
				self.velocity = d
				return
		for d in road_directions:
			cell = here + d
			if cell not in self.backtracked_cells:
				self.backtracked_cells.add(cell)
				self.velocity = d
				return
		#  No unbacktracked road in any direction
		#self.get_stuck()
		self.reset_memory()
		self.find_initial_walking_direction()
	
	def attempt_delivery(self):
		#  Deliver newspapers to nearby houses
		#if self.paper_count > 0:
			city = self.city
			here = self.cell
			ahead = self.velocity
			#print "Runner is looking for houses near", here ###
			for d in delivery_directions(ahead):
				tile = city.tile_at(here + d)
				#print "Runner found", tile, "in direction", d ###
				if tile.is_house and tile.subscription == self.team_no \
					and not tile.newspaper_delivered:
						if self.team.get_copy():
							tile.deliver(self.team.team_no, self.news_value, by_runner = True)
							#self.paper_count -= 1
							#if self.paper_count == 0:
							if not self.team.copies_available():
								self.go_off_duty()

	def go_off_duty(self):
		self.stop()
		self.active = False

#------------------------------------------------------------------------------

def direction_alternatives(ahead):
	left = rotate_anticlockwise(ahead)
	yield left
	yield ahead
	yield -left
	yield -ahead

def delivery_directions(ahead):
	left = rotate_anticlockwise(ahead)
	yield left
	yield -left
	