#------------------------------------------------------------------------------
#
#   Interesting Times - Tile
#
#------------------------------------------------------------------------------

from __future__ import division
from copy import copy
from pygame import Rect, Color
from albow.resource import get_image
from albow.image_array import get_image_array
from probability import random_vector
import constants as k

class Tile(object):
	#  width   int  number of cells wide
	#  height  int  number of cells high
	
	bar_colors = [Color("blue"), Color("red")]

	is_road = False
	is_house = False
	is_newsstand = False
	
	def __init__(self, width, height):
		self.width = width
		self.height = height
	
	irange = property(lambda self: xrange(self.width))
	jrange = property(lambda self: xrange(self.height))
	
	def get_rect(self):
		return Rect(0, 0, self.width * k.CELL_WIDTH, self.height * k.CELL_HEIGHT)
	
	def clone(self):
		return copy(self)
	
	def random_cell(self):
		return self.origin + random_vector((self.width, self.height))
	
	def draw_bar(self, surf, pos, row, nrows, value, color):
		a = self.bar_area
		h = a.height // nrows
		w = int(round(min(value, 1.0) * a.width))
		r = Rect(pos[0] + a.left, pos[1] + a.top + row * h, w, h)
		surf.fill(color, r)
	
	def iter_cells(self, extra_x = 0, extra_y = 0):
		i0, j0 = self.origin
		for i in xrange(i0 - extra_x, i0 + self.width + extra_x):
			for j in xrange(j0 - extra_y, j0 + self.height + extra_y):
				yield (i, j)
	
#------------------------------------------------------------------------------

class NoTileClass(Tile):

	def __nonzero__(self):
		return False

NoTile = NoTileClass(0, 0)

#------------------------------------------------------------------------------

class StaticTile(Tile):
	#  image_name  string

	def __init__(self, width, height, image_name):
		Tile.__init__(self, width, height)
		self.image_name = image_name
	
	def draw(self, surf, pos):
		image = self.get_image()
		surf.blit(image, pos)
	
	def get_image(self):
		return get_tile_image(self.image_name)
	
	def __str__(self):
		return "%s(%s, %r)" % (self.__class__.__name__, self.origin, self.image_name)
		
#------------------------------------------------------------------------------

def get_tile_image(name):
	return get_image("tiles/" + name)

def get_tile_image_array(name, shape):
	return get_image_array("tiles/" + name, shape)
